#include "dspmanager.h"

DSPManager::DSPManager(long int length)
{
    this->length = length;
    generateHammingWindow();
    generateHannWindow();
    generateBlackmanWindow();
    generateBlackmanHarrisWindow();
}

void DSPManager::generateHammingWindow(){
    hamming_window = (double *) malloc(sizeof(double)*length);

    for(int i=0; i < length; ++i)
         hamming_window[i] = 0.53836 - 0.46164*cos(2*PI*i/(length-1.0));
}

void DSPManager::generateHannWindow(){
    hann_window = (double *) malloc(sizeof(double)*length);

    for(int i=0; i < length; ++i)
         hann_window[i] = 0.5*(1-cos(2*PI*i/(length-1)));
}

void DSPManager::generateBlackmanWindow(){
    blackman_window = (double *) malloc(sizeof(double)*length);

    for(int i=0; i < length; ++i)
         blackman_window[i] = 0.42 - 0.5*cos(2*PI*i/(length-1)) + 0.08*cos(4*PI*i/(length-1));
}

void DSPManager::generateBlackmanHarrisWindow(){
    blackmanHarris_window = (double *) malloc(sizeof(double)*length);

    for(int i=0; i < length; ++i)
         blackmanHarris_window[i] = 0.35875 - 0.48829*cos(2*PI*i/(length-1)) + 0.14128*cos(4*PI*i/(length-1)) - 0.01168*cos(6*PI*i/(length-1));
}

void DSPManager::applyWindow(short *data, short window_flag){

    double *t_window = (window_flag == HAMMING)?hamming_window:
                       (window_flag == HANN)?hann_window:
                       (window_flag == BLACKMAN)?blackman_window:
                       (window_flag == BLACKMAN_HARRIS)?blackmanHarris_window:nullptr;

    for(int i = 0; i < length; i++)
        data[i] *= t_window[i];
}

short DSPManager::getMaxima(double *t, int size){
    double min = -1;
    int sample = 0;
    for(int i = 0; i < size/2; i++)
        if(abs(t[i])>min){
            min = t[i];
            sample = i;
        }
    return sample;
}
