#ifndef MICMANAGER_H
#define MICMANAGER_H

#include <string.h>
#include <iostream>
#include <math.h>
#include "alsa/asoundlib.h"
#include "dspmanager.h"

/**
 * @brief Permite la captura de micrófono a través de API de ALSA
 */
class MicManager {
    
    public:
        MicManager(short bufferSizeMatrix, int totalBufferSize, long int sampleFreq);
        /**
         * @brief Leer buffer interno de tarjeta de audio
         */
        void readBuffer();
        /**
         * @brief Cerrar dispositivo de captura
         */
        void closeDev();
        /**
         * @brief Rellenar buffer circular con datos del buffer de tarjeta de audio
         */
        void populateTotalBuffer();
        /**
         * @brief Configurar tamaño del buffer interno de tarjeta de audio
         * @param size tamaño en bytes de buffer
         */
        void setBufferSize(int size);
        /**
         * @brief Configurar tamaño del buffer circular
         * @param size tamaño en bytes de buffer circular
         */
        void setTotalBufferSize(int size);
        /**
         * @brief Configurar frecuencia de muestreo
         * @param sr frecuencia de muestreo
         */
        void setSampleRate(long int sr);
        /**
         * @brief Abrir y ejecutar dispositivo de captura
         */
        void openDev();
        /**
         * @brief Configurar dispositivo de captura
         * @param sr Dispositivo a usar (ver documentacion de ALSA)
         */
        void setDev(int sr);

        /**
         * @brief Buffer que contiene las muestras capturadas por el micrófono
         * @return puntero a buffer interno
         */
        short *getBuffer();
        /**
         * @brief Buffer circular que se rellena constantemente con las muestras del micrófono
         * @return puntero a buffer circular
         */
        short *getTotalBuffer();
        /**
         * @brief Detector de peaks rudimentario
         * @param t puntero a arreglo de muestras a procesar
         * @return posición de muestra máxima
         */
        short getMaxima(double *t);

    private:

        int index = 0;
        int err_code;
        int buffer_size;
        int total_buffer_size;
        long int sample_rate;

        short *buffer;
        short *total_buffer;

        const char *device = "default";
        snd_pcm_t *handle_capture;
        snd_pcm_sframes_t frames;
};

#endif /* MICMANAGER_H */

